"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.laravel = void 0;
var code_builder_1 = require("../../../../helpers/code-builder");
var escape_1 = require("../../../../helpers/escape");
exports.laravel = {
    info: {
        key: 'laravel',
        title: 'Laravel',
        link: 'https://laravel.com/',
        description: 'ReadMe Metrics Webhooks SDK usage on Laravel'
    },
    convert: function (_a, options) {
        var secret = _a.secret, security = _a.security, server = _a.server;
        var opts = __assign({ indent: '    ' }, options);
        var _b = new code_builder_1.CodeBuilder({
            indent: opts.indent
        }), blank = _b.blank, endSection = _b.endSection, join = _b.join, push = _b.push, pushVariable = _b.pushVariable, ranges = _b.ranges, startSection = _b.startSection;
        push('<?php');
        blank();
        push('// Your ReadMe secret');
        push("$secret = '".concat(secret, "';"));
        blank();
        push('// Add this code into your `routes/web.php` file.');
        push("Route::post('/webhook', function (\\Illuminate\\Http\\Request $request) use ($secret) {");
        startSection('verification');
        push('// Verify the request is legitimate and came from ReadMe.', 1);
        push("$signature = $request->headers->get('readme-signature', '');", 1);
        blank();
        push('try {', 1);
        push('\\ReadMe\\Webhooks::verify($request->input(), $signature, $secret);', 2);
        push('} catch (\\Exception $e) {', 1);
        push('// Handle invalid requests', 2);
        push('return response()->json([', 2);
        push("'error' => $e->getMessage()", 3);
        push('], 401);', 2);
        push('}', 1);
        endSection('verification');
        blank();
        startSection('payload');
        push('// Fetch the user from the database and return their data for use with OpenAPI variables.', 1);
        push("// $user = DB::table('users')->where('email', $request->input('email'))->limit(1)->get();", 1);
        push('return response()->json([', 1);
        if (!server.length && !security.length) {
            push('// Add custom data to return in your webhook call here.', 2);
        }
        if (server.length) {
            push('// OAS Server variables', 2);
            server.forEach(function (data) {
                pushVariable("'".concat((0, escape_1.escapeForSingleQuotes)(data.name), "' => '").concat((0, escape_1.escapeForSingleQuotes)(data["default"] || data["default"] === '' ? data["default"] : data.name), "',"), {
                    type: 'server',
                    name: data.name,
                    indentationLevel: 2
                });
            });
        }
        if (server.length && security.length) {
            blank();
        }
        if (security.length) {
            push('// OAS Security variables', 2);
            security.forEach(function (data) {
                if (data.type === 'http') {
                    // Only HTTP Basic auth has any special handling for supplying auth.
                    if (data.scheme === 'basic') {
                        pushVariable("'".concat((0, escape_1.escapeForSingleQuotes)(data.name), "' => ['user' => 'user', 'pass' => 'pass'],"), {
                            type: 'security',
                            name: data.name,
                            indentationLevel: 2
                        });
                        return;
                    }
                }
                pushVariable("'".concat((0, escape_1.escapeForSingleQuotes)(data.name), "' => '").concat((0, escape_1.escapeForSingleQuotes)(data["default"] || data["default"] === '' ? data["default"] : data.name), "',"), {
                    type: 'security',
                    name: data.name,
                    indentationLevel: 2
                });
            });
        }
        push(']);', 1);
        endSection('payload');
        push('});');
        blank();
        return {
            ranges: ranges(),
            snippet: join()
        };
    }
};
